from numpy import dstack
from pandas import read_csv
import numpy as np
from tensorflow import keras
from sklearn.model_selection import train_test_split
from tensorflow.keras.utils import to_categorical

def load_file(filepath):
    dataframe = read_csv(filepath, header=None, delim_whitespace=True)
    return dataframe.values

def load_group(filenames, prefix=''):
    loaded = list()
    for name in filenames:
        data = load_file(prefix + name)
        loaded.append(data)
    # stack group so that features are the 3rd dimension
    loaded = dstack(loaded)
    return loaded
 
def load_dataset_group(group, prefix=''):
    filepath = prefix + group + '/Inertial Signals/'
    # load all 9 files as a single array
    filenames = list()
    # total acceleration
    filenames += ['total_acc_x_'+group+'.txt', 'total_acc_y_'+group+'.txt', 'total_acc_z_'+group+'.txt']
    # body acceleration
    filenames += ['body_acc_x_'+group+'.txt', 'body_acc_y_'+group+'.txt', 'body_acc_z_'+group+'.txt']
    # body gyroscope
    filenames += ['body_gyro_x_'+group+'.txt', 'body_gyro_y_'+group+'.txt', 'body_gyro_z_'+group+'.txt']
    # load input data
    X = load_group(filenames, filepath)
    # load class output
    y = load_file(prefix + group + '/y_'+group+'.txt')
    return X, y

def load_dataset(prefix=''):
    # load all train
    trainX, trainy = load_dataset_group('train', prefix + 'dataset/')
    print(trainX.shape, trainy.shape)

    trainX, valX, trainy, valY = train_test_split(trainX, trainy, test_size=0.2, shuffle= True)
    print(valX.shape, valY.shape)

    # load all test
    testX, testy = load_dataset_group('test', prefix + 'dataset/')
    print(testX.shape, testy.shape)
    # zero-offset class values
    trainy = trainy - 1
    testy = testy - 1
    valY = valY - 1
    # one hot encode y
    trainy = to_categorical(trainy)
    testy = to_categorical(testy)
    valY = to_categorical(valY)
    return np.array(trainX, dtype=np.float32), np.array(trainy, dtype=np.float32), np.array(valX, dtype=np.float32), np.array(valY, dtype=np.float32), np.array(testX, dtype=np.float32), np.array(testy, dtype=np.float32)

def compile_model(model):
    loss_fn = keras.losses.CategoricalCrossentropy(from_logits=False)
    optimizer = keras.optimizers.SGD(learning_rate=1e-2, momentum=0.7)
    model.compile(loss=loss_fn, optimizer=optimizer, metrics=['accuracy'])
    return model

def compute_pruned_count(percent_pruning, layer):
    total = layer.output_shape[1]
    nb_to_be_pruned = int(np.floor(percent_pruning * total))
    return nb_to_be_pruned

def get_weights_l1(layer):
    weights = layer.get_weights()[0]
    num_filter = len(weights[1])
    norms = []
    for i in range(num_filter):
        l1_norm = np.sum(abs(weights[:,i]))
        norms.append(l1_norm)
    return np.array(norms)

def set_group_weights_to_zero(model, layer_id, percentage):  
    diag_weights = model.layers[layer_id].get_weights()[0].diagonal()
    max_s = []
    s = 0
    for i in range(0, diag_weights.shape[0]):
        if (i % 9 != 0):
            s = s + diag_weights[i]
        else:
            max_s.append(s)
            s = diag_weights[i]
    nb_to_be_pruned = int(np.floor(percentage * 128))
    weights_to_be_pruned = (np.asarray(max_s)).argsort()[:nb_to_be_pruned]   
    w = sorted(weights_to_be_pruned)
   
    mask = np.ones((128))
    for j in range (0, len(w)):
        x = w[j]
        mask[x] = 0
  
    weights = model.layers[layer_id].get_weights()[0]      
    for i in range(0, weights_to_be_pruned.shape[0]):
        x = weights_to_be_pruned[i]
        for j in range (0, 9):
            weights[9*x+j, 9*x+j] = 0
    
    new_model = keras.models.clone_model(model)
    new_model.layers[layer_id].set_weights([weights])
    compile_model(new_model)
    return new_model
